From bcf5310ffc57108df391ab4a74f0fc559d78cd92 Mon Sep 17 00:00:00 2001
From: Zhigang Wang <wangzhigang17@huawei.com>
Date: Tue, 12 Jan 2021 11:41:52 +0800
Subject: [PATCH 2/3] interrupt_controller: update construction and realize
 function of GIC-v3

1. Remove redundant arch feature in aarch64 sub-module of
   interrupt_controller module, because arch feature already exists
   in mod.rs of interrupt_controller module.
2. Split GIC-v3's construction function and realize function.

Signed-off-by: Zhigang Wang <wangzhigang17@huawei.com>
---
 device_model/src/interrupt_controller/aarch64/gicv3.rs | 10 ++++++----
 device_model/src/interrupt_controller/aarch64/mod.rs   |  5 +++--
 2 files changed, 9 insertions(+), 6 deletions(-)

diff --git a/device_model/src/interrupt_controller/aarch64/gicv3.rs b/device_model/src/interrupt_controller/aarch64/gicv3.rs
index fbad4db..4753b2e 100644
--- a/device_model/src/interrupt_controller/aarch64/gicv3.rs
+++ b/device_model/src/interrupt_controller/aarch64/gicv3.rs
@@ -380,15 +380,17 @@ impl GICDevice for GICv3 {
         vm: &Arc<VmFd>,
         gic_conf: &GICConfig,
     ) -> Result<Arc<dyn GICDevice + std::marker::Send + std::marker::Sync>> {
-        let gic = GICv3::new(vm, gic_conf)?;
+        Ok(Arc::new(GICv3::new(vm, gic_conf)?))
+    }
 
-        gic.realize()?;
+    fn realize(&self) -> Result<()> {
+        self.realize()?;
 
-        if let Some(its) = &gic.its_dev {
+        if let Some(its) = &self.its_dev {
             its.realize()?;
         }
 
-        Ok(Arc::new(gic))
+        Ok(())
     }
 
     fn generate_fdt(&self, fdt: &mut Vec<u8>) -> errors::Result<()> {
diff --git a/device_model/src/interrupt_controller/aarch64/mod.rs b/device_model/src/interrupt_controller/aarch64/mod.rs
index 1c58301..03c336b 100644
--- a/device_model/src/interrupt_controller/aarch64/mod.rs
+++ b/device_model/src/interrupt_controller/aarch64/mod.rs
@@ -19,7 +19,6 @@ use kvm_ioctls::VmFd;
 pub use gicv3::Error as GICError;
 pub use gicv3::GICv3;
 use machine_manager::machine::{KvmVmState, MachineLifecycle};
-#[cfg(target_arch = "aarch64")]
 use util::{device_tree, errors};
 
 // First 32 are private to each CPU (SGIs and PPIs).
@@ -78,6 +77,9 @@ pub trait GICDevice: MachineLifecycle {
     where
         Self: Sized;
 
+    /// Realize function for kvm_based `GIC` device.
+    fn realize(&self) -> Result<(), GICError>;
+
     /// Constructs `fdt` node for `GIC`.
     ///
     /// # Arguments
@@ -88,7 +90,6 @@ pub trait GICDevice: MachineLifecycle {
 
 /// A wrapper around creating and using a kvm-based interrupt controller.
 pub struct InterruptController {
-    #[cfg(target_arch = "aarch64")]
     gic: Arc<dyn GICDevice + std::marker::Send + std::marker::Sync>,
 }
 
-- 
2.27.0

